@@ -64,10 +64,22 @@ def generate_tessellation(extent_layer=None,
     :returns: FeatureLayer or Feature Layer Collection
 
     """
-    kwargs = locals()
+
     gis = _arcgis.env.active_gis if gis is None else gis
     if not ((context and 'extent' in context) or extent_layer):
         raise ValueError("Tool requires an extent_layer or defined extent.")
+    kwargs = {
+        "extent_layer" : extent_layer,
+        "bin_size" : bin_size,
+        "bin_size_unit" : bin_size_unit,
+        "bin_type" : bin_type,
+        "intersect_study_area" : intersect_study_area,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "future" : future
+    }
     params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.generate_tessellations, **kwargs)
     if extent_layer is None:
         params['extent_layer'] = None
@@ -187,8 +199,19 @@ def dissolve_boundaries(
                                             output_name="DissolveBoundaries")
     """
 
-    kwargs = locals()
+
     gis = _arcgis.env.active_gis if gis is None else gis
+    kwargs = {
+        "input_layer" : input_layer,
+        "dissolve_fields" : dissolve_fields,
+        "summary_fields" : summary_fields,
+        "multi_part_features" : multi_part_features,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "future" : future
+    }
     params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.dissolve_boundaries, **kwargs)
     return gis._tools.featureanalysis.dissolve_boundaries(**params)
 #----------------------------------------------------------------------
@@ -268,8 +291,18 @@ def extract_data(
                                  data_format='ShapeFile',
                                  output_name='state highway extracted')
     """
-    kwargs = locals()
     gis = _arcgis.env.active_gis if gis is None else gis
+    kwargs = {
+        "input_layers" : input_layers,
+        "extent" : extent,
+        "clip" : clip,
+        "data_format" : data_format,
+        "output_name" : output_name,
+        "gis" : gis,
+        "estimate" : estimate,
+        "future" : future
+    }
+
     params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.extract_data, **kwargs)
     return gis._tools.featureanalysis.extract_data(**params)
 #----------------------------------------------------------------------
@@ -348,8 +381,18 @@ def merge_layers(
                               merging_attributes=["State Match Place_Name"],
                               output_name="merge layers")
     """
-    kwargs = locals()
+
     gis = _arcgis.env.active_gis if gis is None else gis
+    kwargs = {
+        "input_layer" : input_layer,
+        "merge_layer" : merge_layer,
+        "merging_attributes" : merging_attributes,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "future" : future
+    }
     params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.merge_layers, **kwargs)
     return gis._tools.featureanalysis.merge_layers(**params)
 #----------------------------------------------------------------------
@@ -458,8 +501,20 @@ def overlay_layers(
 
 
     """
-    kwargs = locals()
     gis = _arcgis.env.active_gis if gis is None else gis
+    kwargs = {
+        "input_layer" : input_layer,
+        "overlay_layer" : overlay_layer,
+        "overlay_type" : overlay_type,
+        "snap_to_input" : snap_to_input,
+        "output_type" : output_type,
+        "tolerance" : tolerance,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "future" : future
+    }
     params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.overlay_layers, **kwargs)
     return gis._tools.featureanalysis.overlay_layers(**params)
 #----------------------------------------------------------------------
@@ -530,8 +585,19 @@ def create_route_layers(route_data_item,
                             route_name_prefix="santa_ana",
                             folder_name="create route layers")
     """
-    kwargs = locals()
+
     gis = _arcgis.env.active_gis if gis is None else gis
+    kwargs = {
+        "route_data_item" : route_data_item,
+        "delete_route_data_item" : delete_route_data_item,
+        "tags" : tags,
+        "summary" : summary,
+        "route_name_prefix" : route_name_prefix,
+        "folder_name" : folder_name,
+        "gis" : gis,
+        "estimate" : estimate,
+        "future" : future
+    }
     params_tool = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.create_route_layers, **kwargs)
     params = inspect_function_inputs(fn=gis._tools.featureanalysis.create_route_layers, **kwargs)
     if 'context' not in params_tool and 'context' in params:
@@ -563,7 +629,3 @@ def create_route_layers(route_data_item,
     if output_name:
         params['output_name'] = output_name
     return gis._tools.featureanalysis.create_route_layers(**params)
-    #return gis._tools.featureanalysis.create_route_layers(
-        #route_data_item,
-        #delete_route_data_item,
-        #output_name, estimate=estimate, future=future)
