@@ -213,10 +213,29 @@ def connect_origins_to_destinations(origins_layer,
                                          time_of_day=datetime(1990, 1, 4, 1, 3),
                                          output_name="routes_from_offices_to_hq")
     """
-    kwargs = locals()
+
     gis = _arcgis.env.active_gis if gis is None else gis
-    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.connect_origins_to_destinations, 
-                                     **kwargs)    
+    kwargs = {
+        "origins_layer" : origins_layer,
+        "destinations_layer" : destinations_layer,
+        "measurement_type" : measurement_type,
+        "origins_layer_route_id_field" : origins_layer_route_id_field,
+        "destinations_layer_route_id_field" : destinations_layer_route_id_field,
+        "time_of_day" : time_of_day,
+        "time_zone_for_time_of_day" : time_zone_for_time_of_day,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "point_barrier_layer" : point_barrier_layer,
+        "line_barrier_layer" : line_barrier_layer,
+        "polygon_barrier_layer"  : polygon_barrier_layer,
+        "future" : future,
+        "route_shape" : route_shape,
+        "include_route_layers" : include_route_layers
+    }
+    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.connect_origins_to_destinations,
+                                     **kwargs)
 
     if isinstance(measurement_type, str):
         route_service = network.RouteLayer(gis.properties.helperServices.route.url, gis=gis)
@@ -230,20 +249,19 @@ def connect_origins_to_destinations(origins_layer,
 
     return gis._tools.featureanalysis.connect_origins_to_destinations(**params)
 #--------------------------------------------------------------------------
-def create_buffers(
-    input_layer,
-        distances=[],
-        field=None,
-        units="Meters",
-        dissolve_type="None",
-        ring_type="Disks",
-        side_type="Full",
-        end_type="Round",
-        output_name=None,
-        context=None,
-        gis=None,
-        estimate=False,
-        future=False):
+def create_buffers(input_layer,
+                   distances=[],
+                   field=None,
+                   units="Meters",
+                   dissolve_type="None",
+                   ring_type="Disks",
+                   side_type="Full",
+                   end_type="Round",
+                   output_name=None,
+                   context=None,
+                   gis=None,
+                   estimate=False,
+                   future=False):
     """
     .. image:: _static/images/create_buffers/create_buffers.png
 
@@ -381,10 +399,25 @@ def create_buffers(
                                  output_name='create_buffers',
                                  context={"extent":{"xmin":-12555831.656684224,"ymin":5698027.566358956,"xmax":-11835489.102124758,"ymax":6104672.556836072,"spatialReference":{"wkid":102100,"latestWkid":3857}}})
     """
-    kwargs = locals()
+
     gis = _arcgis.env.active_gis if gis is None else gis
-    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.create_buffers, 
-                                     **kwargs)    
+    kwargs = {
+        "input_layer" : input_layer,
+        "distances" : distances,
+        "field" : field,
+        "units" : units,
+        "dissolve_type" : dissolve_type,
+        "ring_type" : ring_type,
+        "side_type" : side_type,
+        "end_type" : end_type,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "future" : future
+    }
+    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.create_buffers,
+                                     **kwargs)
     return gis._tools.featureanalysis.create_buffers(**params)
 #--------------------------------------------------------------------------
 def create_drive_time_areas(input_layer,
@@ -594,10 +627,30 @@ def create_drive_time_areas(input_layer,
                                        output_name='create_drive_time_areas',
                                        context={"extent":{"xmin":-11134400.655784884,"ymin":3368261.7800108367,"xmax":-10682810.692676282,"ymax":3630899.409198575,"spatialReference":{"wkid":102100,"latestWkid":3857}}}) """
 
-    kwargs = locals()
+
     gis = _arcgis.env.active_gis if gis is None else gis
-    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.create_drive_time_areas, 
-                                     **kwargs)    
+    kwargs = {
+        "input_layer" : input_layer,
+        "break_values" : break_values,
+        "break_units" : break_units,
+        "travel_mode" : travel_mode,
+        "overlap_policy" : overlap_policy,
+        "time_of_day" : time_of_day,
+        "time_zone_for_time_of_day" : time_zone_for_time_of_day,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "point_barrier_layer" : point_barrier_layer,
+        "line_barrier_layer" : line_barrier_layer,
+        "polygon_barrier_layer" : polygon_barrier_layer,
+        "future" : future,
+        "travel_direction" : travel_direction,
+        "show_holes" : show_holes,
+        "include_reachable_streets" : include_reachable_streets
+    }
+    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.create_drive_time_areas,
+                                     **kwargs)
 
     if isinstance(travel_mode, str):
         route_service = network.RouteLayer(gis.properties.helperServices.route.url, gis=gis)
@@ -788,10 +841,30 @@ def find_nearest(
                                include_route_layers=True,
                                point_barrier_layer=road_closures_lyr))
     """
-    kwargs = locals()
     gis = _arcgis.env.active_gis if gis is None else gis
-    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.find_nearest, 
-                                     **kwargs)    
+    kwargs = {
+        "analysis_layer" : analysis_layer,
+        "near_layer" : near_layer,
+        "measurement_type" : measurement_type,
+        "max_count" : max_count,
+        "search_cutoff" : search_cutoff,
+        "search_cutoff_units" : search_cutoff_units,
+        "time_of_day" : time_of_day,
+        "time_zone_for_time_of_day" : time_zone_for_time_of_day,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "include_route_layers" : include_route_layers,
+        "point_barrier_layer" : point_barrier_layer,
+        "line_barrier_layer" : line_barrier_layer,
+        "polygon_barrier_layer" : polygon_barrier_layer,
+        "future" : future
+    }
+
+    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.find_nearest,
+                                     **kwargs)
+    params['estimate'] = estimate
     if isinstance(measurement_type, str):
         route_service = network.RouteLayer(gis.properties.helperServices.route.url, gis=gis)
         travelmodes = route_service.retrieve_travel_modes()
@@ -803,8 +876,7 @@ def find_nearest(
                 params['measurement_type'] = measurement_type
     params['time_of_day'] = _date_handler(time_of_day)
     return gis._tools.featureanalysis.find_nearest(**params)
-
-
+#--------------------------------------------------------------------------
 def plan_routes(
     stops_layer,
         route_count,
@@ -1085,10 +1157,35 @@ def plan_routes(
                             output_name='plan route for employees')
 
     """
-    kwargs = locals()
+
     gis = _arcgis.env.active_gis if gis is None else gis
-    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.plan_routes, 
-                                     **kwargs)    
+    kwargs = {
+        "stops_layer" : stops_layer,
+        "route_count" : route_count,
+        "max_stops_per_route" : max_stops_per_route,
+        "route_start_time" : route_start_time,
+        "start_layer" : start_layer,
+        "start_layer_route_id_field" : start_layer_route_id_field,
+        "return_to_start" : return_to_start,
+        "end_layer" : end_layer,
+        "end_layer_route_id_field" : end_layer_route_id_field,
+        "travel_mode" : travel_mode,
+        "stop_service_time" : stop_service_time,
+        "max_route_time" : max_route_time,
+        "include_route_layers" : include_route_layers,
+        "output_name" : output_name,
+        "context" : context,
+        "gis" : gis,
+        "estimate" : estimate,
+        "point_barrier_layer" : point_barrier_layer,
+        "line_barrier_layer" : line_barrier_layer,
+        "polygon_barrier_layer" : polygon_barrier_layer,
+        "future" : future
+    }
+    params = inspect_function_inputs(fn=gis._tools.featureanalysis._tbx.plan_routes,
+                                     **kwargs)
+    params['estimate'] = estimate
+
     if isinstance(travel_mode, str):
         route_service = network.RouteLayer(gis.properties.helperServices.route.url, gis=gis)
         travelmodes = route_service.retrieve_travel_modes()
