@@ -102,7 +102,7 @@ class Geocoder(_GISResource):
                                  a user will search for places and addresses only
                                  within the current map extent.
         --------------------     ----------------------------------------------------
-        location                 optionl [x,y], Defines an origin point location that
+        location                 optional [x,y], Defines an origin point location that
                                  is used with the distance parameter to sort
                                  geocoding candidates based upon their proximity to
                                  the location.
@@ -396,7 +396,7 @@ class Geocoder(_GISResource):
         if preferred_label_values is not None:
             params['preferredLabelValues'] = preferred_label_values
 
-        resp = self._con.post(url, params, token=self._token)
+        resp = self._con.post(url, params)
         if resp is not None and as_featureset:
             sr = resp['spatialReference']
 
@@ -1012,7 +1012,7 @@ def geocode(address,
                              a user will search for places and addresses only
                              within the current map extent.
     --------------------     ----------------------------------------------------
-    location                 Optionl [x,y], Defines an origin point location that
+    location                 Optional [x,y], Defines an origin point location that
                              is used with the distance parameter to sort
                              geocoding candidates based upon their proximity to
                              the location.
