@@ -343,6 +343,8 @@ def _execute_gp_tool(gis, task_name, params, param_db, return_values, use_async,
                     elif  _is_geoenabled(param_value):
                         gp_params[gp_param_name] = json.loads(json.dumps(param_value.spatial.__feature_set__,
                                                                          default=_date_handler))
+                    elif isinstance(param_value, arcgis.gis.Layer):
+                        gp_params[gp_param_name] = _layer_input_gp(param_value)
                     elif type(param_value) == str:
 
                         try:
@@ -427,7 +429,10 @@ def _execute_gp_tool(gis, task_name, params, param_db, return_values, use_async,
     else: # synchronous
         exec_url = url + "/" + task_name + "/execute"
         if add_token:
-            resp = gptool._con.post(exec_url, gp_params, token=gptool._token)
+            try:
+                resp = gptool._con.post(exec_url, gp_params, token=gptool._token)
+            except:
+                resp = gptool._con.post(exec_url, gp_params)
         else:
             resp = gptool._con.post(exec_url, gp_params)
 
