@@ -25,6 +25,7 @@ try:
     from fastai.vision.data import imagenet_stats
     from fastai.core import subplots
     from fastai.data_block import DataBunch
+    from fastai.vision.data import ImageDataBunch
     from fastai.torch_core import grab_idx, to_detach
     import torchvision.transforms.functional as TF
     from typing import Sequence
@@ -136,7 +137,7 @@ class DiscreteAffine:
 
 
 class RoadOrientation():
-    def __init__(self, classified_tiles_data: ArcgisData, **kwargs):
+    def __init__(self,orig_data, classified_tiles_data: ArcgisData, **kwargs):
         """
         The class is used to create Fast AI Databunch for Multi-Task
         Road - Orientation learning. It internally utilizes RoadOrientation
@@ -181,13 +182,18 @@ class RoadOrientation():
         self.multi_scale = kwargs.get("multi_scale", default_road_params['multi_scale'])
 
         self.files = []
-        search_dir = os.path.join(self.base.img_root, "*.{}".format(self.base.extension[0]))
-        for image_file in glob.glob(search_dir):
-            image_file = Path(image_file)
-            image_name = image_file.stem
+        for cnt, image in enumerate(orig_data.train_ds.x.items):
             file_pair = {
-                "image": os.path.join(self.base.img_root, f"{image_name}.{self.base.extension[0]}"),
-                "label": os.path.join(self.base.gt_root, f"{image_name}.{self.base.extension[1]}"),
+                "image": image,
+                "label": orig_data.train_ds.y.items[cnt],
+            }
+            self.files.append(file_pair)
+
+            # self.valid_files = []
+        for cnt, image in enumerate(orig_data.valid_ds.x.items):
+            file_pair = {
+                "image": image,
+                "label": orig_data.valid_ds.y.items[cnt],
             }
             self.files.append(file_pair)
 
@@ -234,7 +240,7 @@ class RoadOrientation():
             ]
         ] if self.base.transforms is None else self.base.transforms
 
-    def get_databunch(self,orig_data,**kwargs) -> DataBunch:
+    def get_databunch(self,orig_data,**kwargs) -> ImageDataBunch:
         """
         Method to create Databunch
         """
@@ -252,7 +258,7 @@ class RoadOrientation():
         )
 
         device = get_device()
-        data = DataBunch(train_dl, valid_dl, device=device)
+        data = ImageDataBunch(train_dl, valid_dl, device=device)
 
 
         data.chip_size = data.train_ds[0][0].shape[-1]
