@@ -8,12 +8,12 @@ import tempfile
 from pathlib import Path
 HAS_FASTAI = False
 try:
-    from .env import raise_fastai_import_error, HAS_GDAL, gdal_import_exception, GDAL_INSTALL_MESSAGE, LAMBDA_TEXT_CLASSIFICATION
-    if not LAMBDA_TEXT_CLASSIFICATION:
+    from .env import raise_fastai_import_error, HAS_GDAL, gdal_import_exception, GDAL_INSTALL_MESSAGE, _LAMBDA_TEXT_CLASSIFICATION
+    if not _LAMBDA_TEXT_CLASSIFICATION:
         from fastai.vision.data import ImageList
         from fastai.vision import Image, imagenet_stats, pil2tensor
         import PIL
-    elif LAMBDA_TEXT_CLASSIFICATION:
+    elif _LAMBDA_TEXT_CLASSIFICATION:
         missing_classes = ['Image', 'ImageList']
         for missing_class in missing_classes: 
             exec(f'{missing_class} = object')
@@ -195,7 +195,18 @@ class ArcGISMSImage(Image):
             if len(x.shape)==2:
                 x = x.unsqueeze(0)
             if div is not None:
-                x = x / div
+                if isinstance(div, tuple):
+                    min_values, max_values = div
+                    if not isinstance(min_values, torch.Tensor):
+                        min_values, max_values = torch.tensor(min_values), torch.tensor(max_values)
+                    for i in range(x.shape[0]):
+                        arr = x[i,:,:]
+                        arr[torch.isnan(arr)] = min_values[i]
+                    min_values = min_values.to(x).view(-1, 1, 1)
+                    max_values = max_values.to(x).view(-1, 1, 1)
+                    x = (x-min_values) / (max_values-min_values+1e-04)
+                else:
+                    x = x / div
 
         return cls(x)
 
@@ -342,7 +353,12 @@ def denorm_image(imagetensor_batch, mean=None, std=None):
 
 def predict_batch(self, imagetensor_batch):
     if self._backend == 'pytorch':
-        predictions = self.learn.model.eval()(imagetensor_batch.to(self._device).float()).detach()
+        predictions = self.learn.model.eval()(imagetensor_batch.to(self._device).float())
+        if type(predictions) in [tuple, list]:
+            predictions = [x.detach() for x in predictions]
+        else:
+            predictions = predictions.detach()
+        #predictions = self.learn.model.eval()(imagetensor_batch.to(self._device).float()).detach()
         return predictions
     elif self._backend == 'tensorflow':
         from .common_tf import predict_batch_tf
@@ -353,7 +369,7 @@ def predict_batch(self, imagetensor_batch):
 ## Image Stretching Functions start ##
 
 def get_band_percent_minmax(values, min_clip, max_clip):
-    return values[round(values.shape[0] * min_clip)], values[values.shape[0] - round(values.shape[0] * max_clip)]
+    return values[math.floor(values.shape[0] * min_clip)], values[values.shape[0] - math.ceil(values.shape[0] * max_clip)]
 
 def get_percent_minmax(imagetensor_batch, min_clip=0.0025, max_clip=0.005):
     shp = imagetensor_batch.shape
