@@ -651,7 +651,7 @@ def prepare_captioning_dataset(path,
 
     # create Databunch
     device = get_device()
-    data = DataBunch(train_dl, valid_dl, device=device, collate_fn=collate_fn)
+    data = DataBunch(train_dl, valid_dl, device=device, collate_fn=collate_fn, path=path)
     # Attach show batch to databunch
     data.show_batch = types.MethodType(show_batch, data)
     # attach vocab
@@ -709,7 +709,7 @@ def show_results(self, rows, **kwargs):
             imsize=5
             )    
     plt.subplots_adjust(top=top)
-    fig.suptitle('Image Before / Image After / Ground Truth / Prediction', fontsize=16)
+    fig.suptitle('Ground Truth / Prediction', fontsize=16)
     nbatches = (rows // self._data.valid_dl.batch_size) + 1
     dls = get_nbatches(self._data.valid_dl, nbatches)
     images, captions = [], []
