@@ -8,6 +8,7 @@ from fastai.core import split_kwargs_by_func
 from .common import ArcGISMSImage, get_nbatches, denorm_x, dynamic_range_adjustment, image_batch_stretcher
 from matplotlib import pyplot as plt
 from matplotlib import patheffects
+from .._utils.env import _IS_ARCGISPRONOTEBOOK
 
 
 class ObjectDetectionCategoryList(ObjectCategoryList):
@@ -106,7 +107,7 @@ def show_batch_pascal_voc_rectangles(self, rows=3, alpha=1, **kwargs): # paramet
         symbology_bands.append(b_index)
 
     # Get Batch
-    x_batch, y_batch = get_nbatches(data_loader, n_items)
+    x_batch, y_batch = get_nbatches(data_loader, math.ceil(n_items/self.batch_size))
     x_batch = torch.cat(x_batch)
     # Denormalize X
     x_batch = (self._scaled_std_values[self._extract_bands].view(1, -1, 1, 1).to(x_batch) * x_batch ) + self._scaled_mean_values[self._extract_bands].view(1, -1, 1, 1).to(x_batch)
@@ -171,6 +172,8 @@ def show_batch_pascal_voc_rectangles(self, rows=3, alpha=1, **kwargs): # paramet
             else:
                 axi.axis('off')
             idx+=1
+    if _IS_ARCGISPRONOTEBOOK:
+        plt.show()
 
 def show_results_multispectral(self, nrows=5, alpha=1, **kwargs): # parameters adjusted in kwargs
     from matplotlib import pyplot as plt
@@ -213,7 +216,7 @@ def show_results_multispectral(self, nrows=5, alpha=1, **kwargs): # parameters a
     label_font_size = kwargs.get('label_font_size', 16)
     
     # Get Batch
-    x_batch, y_batch = get_nbatches(data_loader, nrows)
+    x_batch, y_batch = get_nbatches(data_loader, math.ceil(nrows/self._data.batch_size))
     x_batch = torch.cat(x_batch)
     y_bboxes = []
     y_classes = []
@@ -380,4 +383,6 @@ def show_results_multispectral(self, nrows=5, alpha=1, **kwargs): # parameters a
                     ax_prediction.text(xs[0]+1, ys[0]+1+(label_font_size*(x_batch.shape[-1]-1)/256), lblp, size=label_font_size, color=color, path_effects=[patheffects.Stroke(linewidth=1, foreground='black'), patheffects.Normal()])
             
         idx+=1
+    if _IS_ARCGISPRONOTEBOOK:
+        plt.show()
     return fig,axs
