@@ -45,7 +45,7 @@ class DummyTransform(object):
     def fit_transform(self, x):
         return x
 
-    def inverse_trasnform(self, x):
+    def inverse_transform(self, x):
         return x
 
 
@@ -329,6 +329,9 @@ class TabularDataObject(object):
         if self._is_empty:
             return None
 
+        if self._procs is not None and not isinstance(self._procs, list):
+            self._procs = []
+
         return TabularDataObject._prepare_databunch(
             self._dataframe,
             self._field_mapping,
@@ -365,11 +368,11 @@ class TabularDataObject(object):
                 SimpleImputer(strategy='constant')
             )
 
-            _procs = make_column_transformer(
+            self._procs = make_column_transformer(
                 (numerical_transformer, self._continuous_variables),
                 (categorical_transformer, self._categorical_variables))
-        else:
-            _procs = self._procs
+
+        _procs = self._procs
 
         self._encoder_mapping = None
         if self._categorical_variables:
@@ -636,7 +639,7 @@ class TabularDataObject(object):
 
         return data
 
-    def _process_data(self, dataframe):
+    def _process_data(self, dataframe, fit=True):
         if not HAS_NUMPY:
             raise Exception("This module requires numpy.")
 
@@ -652,24 +655,41 @@ class TabularDataObject(object):
                 SimpleImputer(strategy='constant')
             )
 
-            _procs = make_column_transformer(
+            self._procs = make_column_transformer(
                 (numerical_transformer, self._continuous_variables),
                 (categorical_transformer, self._categorical_variables))
-        else:
-            _procs = self._procs
+
+        _procs = self._procs
 
         if self._encoder_mapping:
             for variable, encoder in self._encoder_mapping.items():
                 dataframe[variable] = np.array(encoder.fit_transform(dataframe[variable]), dtype='int64')
 
-        processed_data = _procs.fit_transform(dataframe)
+        if fit:
+            processed_data = _procs.fit_transform(dataframe)
+        else:
+            processed_data = _procs.transform(dataframe)
 
         return processed_data
 
     def show_batch(self, rows=5, graph=False, seq_len=None):
         """
-        Shows a batch of dataframe prepared without applying transforms.
-        In case of raster only, seq_len = number of rasters
+        Shows a chunk of data prepared without applying transforms.
+        =====================   ===========================================
+        **Argument**            **Description**
+        ---------------------   -------------------------------------------
+        rows                    Optional integer. Number of rows of dataframe
+                                or graph to plot. This parameter is not used
+                                when plotting complete data.
+        ---------------------   -------------------------------------------
+        graph                   Optional boolean. Used for visualizing
+                                time series data. The index_field passed
+                                in prepare_tabulardata is used on the x-axis.
+                                Use this option to plot complete data.
+        ---------------------   -------------------------------------------
+        seq_len                 Optional integer. Used for visualizing data
+                                in the form of graph of seq_len duration.
+        =====================   ===========================================
         """
 
         if seq_len is not None or graph is True:
@@ -740,20 +760,19 @@ class TabularDataObject(object):
             else:
                 x = [i for i in range(len(self._dataframe[self._dependent_variable]))]
 
-            plt.figure(figsize=(25, 5))
-
+            fig, axs = plt.subplots(len(list(self._dataframe.columns)), 1, figsize=(25, 5*len(list(self._dataframe.columns))))
+            counter = 0
             for col in list(self._dataframe.columns):
-                plt.plot(x, self._dataframe[col], label=col)
+                if isinstance(axs, np.ndarray):
+                    axs[counter].plot(x, self._dataframe[col], label=col)
+                    axs[counter].set_title(col)
+                    axs[counter].tick_params(axis="x", labelrotation=60)
+                else:
+                    axs.plot(x, self._dataframe[col], label=col)
+                    axs.set_title(col)
+                    axs.tick_params(axis="x", labelrotation=60)
+                counter = counter + 1
 
-            # naming the x axis
-            plt.xlabel(x_field)
-            # naming the y axis
-            plt.ylabel(self._dependent_variable)
-            plt.tick_params(axis="x", labelrotation=85)
-            # giving a title to my graph
-            plt.title('Data')
-            plt.legend(loc='upper right')
-            # function to show the plot
             plt.show()
 
     @staticmethod
@@ -1128,5 +1147,6 @@ class TabularDataObject(object):
         class_object._encoder_mapping = encoder_mapping
         class_object._is_empty = True
         class_object._procs = procs
+        class_object.path = Path(os.path.abspath('.'))
 
         return class_object
