@@ -1,6 +1,6 @@
-from .._utils.env import LAMBDA_TEXT_CLASSIFICATION
+from .._utils.env import _LAMBDA_TEXT_CLASSIFICATION
 
-if not LAMBDA_TEXT_CLASSIFICATION:
+if not _LAMBDA_TEXT_CLASSIFICATION:
     from ._ssd import SingleShotDetector
     from ._inferencing import _DynamicSSD
     from ._unet import UnetClassifier
@@ -24,4 +24,5 @@ if not LAMBDA_TEXT_CLASSIFICATION:
     from ._multi_task_road_extractor import MultiTaskRoadExtractor
     from ._change_detector import ChangeDetector
     from ._pix2pix import Pix2Pix
-
+    from ._siammask import SiamMask, Track
+    from ._connect_net import ConnectNet
