@@ -1,12 +1,15 @@
 import arcgis
 from pathlib import Path
 import os
+import glob
+import shutil
 import time
 import tempfile
 import json
 import logging
 from .._data import _raise_fastai_import_error
-from .._utils.env import HAS_TENSORFLOW, raise_tensorflow_import_error, LAMBDA_TEXT_CLASSIFICATION
+from .._utils.env import HAS_TENSORFLOW, raise_tensorflow_import_error, _LAMBDA_TEXT_CLASSIFICATION, \
+    _IS_ARCGISPRONOTEBOOK
 from warnings import warn
 import contextlib
 import io
@@ -15,12 +18,14 @@ import socket
 from functools import wraps
 import traceback
 import inspect
+import types
+import functools
 
 HAS_FASTAI = True
 HAS_TENSORBOARDX = True
 
 try:
-    if not LAMBDA_TEXT_CLASSIFICATION:
+    if not _LAMBDA_TEXT_CLASSIFICATION:
         from fastai.vision.learner import model_meta, _default_meta
         from .._utils.common import get_post_processed_model
         from torchvision import models
@@ -42,6 +47,9 @@ try:
     from fastai.basics import partial
     import pandas as pd
     from ... import __version__ as ArcGISLearnVersion
+    from ._pointcnn_utils import AverageMetric
+    from fastai.core import camel2snake
+
 except ImportError as e:
     import_exception = "\n".join(traceback.format_exception(type(e), e, e.__traceback__))
     HAS_FASTAI = False
@@ -69,7 +77,7 @@ losses_skipped = 5
 trailing_losses_skipped = 5
 model_characteristics_folder = 'ModelCharacteristics'
 
-if HAS_FASTAI and not LAMBDA_TEXT_CLASSIFICATION:
+if HAS_FASTAI and not _LAMBDA_TEXT_CLASSIFICATION:
     # Declare the family of backbones to be unpacked and used by different models as supported types
     _vgg_family = [models.vgg11.__name__, models.vgg11_bn.__name__, models.vgg13.__name__, models.vgg13_bn.__name__,
                    models.vgg16.__name__, models.vgg16_bn.__name__, models.vgg19.__name__, models.vgg19_bn.__name__]
@@ -87,6 +95,17 @@ def nostdout():
     sys.stdout = save_stdout
 
 
+def _get_device():
+    if getattr(arcgis.env, "_processorType", "") == "GPU" and torch.cuda.is_available():
+        device = torch.device("cuda")
+    elif getattr(arcgis.env, "_processorType", "") == "CPU":
+        device = torch.device("cpu")
+    else:
+        device = torch.device('cuda') if torch.cuda.is_available() else torch.device('cpu')
+
+    return device
+
+
 class _EmptyDS(object):
     def __init__(self, size):
         self.size = (size, size)
@@ -94,7 +113,7 @@ class _EmptyDS(object):
 
 class _EmptyData():
     def __init__(self, path, c, loss_func, chip_size, train_ds=True):
-        self.path = path
+        self.path = path.parent
         if getattr(arcgis.env, "_processorType", "") == "GPU" and torch.cuda.is_available():
             self.device = torch.device("cuda")
         elif getattr(arcgis.env, "_processorType", "") == "CPU":
@@ -199,16 +218,27 @@ class SaveModelCallback(TrackerCallback):
 
     def on_epoch_end(self, epoch, **kwargs):
         "Compare the value monitored to its best score and maybe save the model."
-
+        current = self.get_monitor_value()
+        if isinstance(current, torch.Tensor):
+            if current.is_cuda:
+                current = current.cpu()
+        self.current = current
         if self.every == "epoch":
-            self.model.save('{}_{}'.format(self.name, epoch), compute_metrics=False)
+            self.model._save(f'{self.name}_epoch_{epoch}', zip_files=False, save_html=False, compute_metrics=False)
         else:  # every="improvement"
-            current = self.get_monitor_value()
             if current is not None and self.operator(current, self.best):
                 if arcgis.env.verbose:
                     print('saving checkpoint.')
+                self.best_epoch = epoch
                 self.best = current
-                self.model._save('{}'.format(self.name), zip_files=False, save_html=False, compute_metrics=False)
+                self.remove_previous()
+                self.model._save(f'{self.name}_epoch_{epoch}', zip_files=False, save_html=False, compute_metrics=False)
+
+    def remove_previous(self):
+        # to avoid creating multiple best checkpoints.
+        saved_path = os.path.join(self.model.learn.path, self.model.learn.model_dir)
+        for p in glob.glob(os.path.join(saved_path, self.name + '*')):
+            shutil.rmtree(p)
 
     def on_train_end(self, **kwargs):
         "Load the best model."
@@ -221,7 +251,7 @@ class SaveModelCallback(TrackerCallback):
                 pass
 
             try:
-                self.model.save('{}'.format(self.name), compute_metrics=False)
+                self.model.save(f'{self.name}_epoch_{self.best_epoch}', compute_metrics=False)
             except:
                 pass
 
@@ -401,7 +431,7 @@ class ArcGISModel(object):
     def __init__(self, data, backbone=None, **kwargs):
         if not HAS_FASTAI:
             _raise_fastai_import_error(import_exception=import_exception)
-        if not LAMBDA_TEXT_CLASSIFICATION:
+        if not _LAMBDA_TEXT_CLASSIFICATION:
             move_to_cpu = _device_check()
         else:
             move_to_cpu = True
@@ -451,6 +481,8 @@ class ArcGISModel(object):
         if not hasattr(data, 'class_mapping') and hasattr(data, 'classes'):
             data.class_mapping = {v: v for v in data.classes}
 
+        if data is not None and getattr(data, 'path', None) is None:
+            data.path = Path(os.path.abspath('.'))
         self.learn = None
         self._data = data
         self._learning_rate = None
@@ -531,6 +563,8 @@ class ArcGISModel(object):
         Helps in choosing the optimum learning rate for training the model.
         """
         self._check_requisites()
+        temp = self.learn.model_dir
+        self.learn.model_dir = os.path.basename(self._data._temp_folder.name)
         try:
             metrics = self.learn.metrics
             self.learn.metrics = []
@@ -539,7 +573,17 @@ class ArcGISModel(object):
             # if some error comes in lr_find
             raise e
         finally:
+            self.learn.model_dir = temp
             self.learn.metrics = metrics
+            tmp_file = os.path.join(self.learn.path, os.path.basename(self._data._temp_folder.name), 'tmp.pth')
+            try:
+                if os.path.exists(tmp_file):
+                    os.remove(tmp_file)
+            except Exception as e:
+                raise e
+
+        #
+        self.learn.model_dir = temp
 
         from IPython.display import clear_output
         clear_output()
@@ -610,7 +654,32 @@ class ArcGISModel(object):
     def _model_metrics(self):
         raise NotImplementedError
 
-    def fit(self, epochs=10, lr=None, one_cycle=True, early_stopping=False, checkpoint=True, tensorboard=False,
+    @property
+    def available_metrics(self):
+        """
+        List of available metrics that are displayed in the training
+        table. Set `monitor` value to be one of these while calling
+        the `fit` method.
+        """
+        metrics = ['valid_loss']
+        for m in self.learn.metrics:
+            if isinstance(m, AverageMetric) or isinstance(m, functools.partial):
+                metrics.append(m.func.__name__)
+            elif isinstance(m, types.FunctionType):
+                metrics.append(m.__name__)
+            else:
+                metrics.append(camel2snake(m.__class__.__name__))
+
+        return metrics
+
+    def fit(self,
+            epochs=10,
+            lr=None,
+            one_cycle=True,
+            early_stopping=False,
+            checkpoint=True,  # "all", "best", True, False ("best" and True are same.)
+            tensorboard=False,
+            monitor='valid_loss',  # whatever is passed here, earlystopping and checkpointing will use that.
             **kwargs):
         """
         Train the model for the specified number of epochs and using the
@@ -632,13 +701,19 @@ class ArcGISModel(object):
                                 learning rate schedule is used.
         ---------------------   -------------------------------------------
         early_stopping          Optional boolean. Parameter to add early stopping.
-                                If set to 'True' training will stop if validation
-                                loss stops improving for 5 epochs.
+                                If set to 'True' training will stop if parameter
+                                `monitor` value stops improving for 5 epochs.
+                                A minimum difference of 0.001 is required for
+                                it to be considered an improvement.
         ---------------------   -------------------------------------------
-        checkpoint              Optional boolean. Parameter to save the best model
-                                during training. If set to `True` the best model
-                                based on validation loss will be saved during
-                                training.
+        checkpoint              Optional boolean or string.
+                                Parameter to save checkpoint during training.
+                                If set to `True` the best model
+                                based on `monitor` will be saved during
+                                training. If set to 'all', all checkpoints
+                                are saved. If set to False, checkpointing will
+                                be off. Setting this parameter loads the best
+                                model at the end of training.
         ---------------------   -------------------------------------------
         tensorboard             Optional boolean. Parameter to write the training log.
                                 If set to 'True' the log will be saved at
@@ -647,6 +722,13 @@ class ArcGISModel(object):
 
                                 The default value is 'False'.
                                 **Note - Not applicable for Text Models
+        ---------------------   -------------------------------------------
+        monitor                 Optional string. Parameter specifies
+                                which metric to monitor while checkpointing
+                                and early stopping. Defaults to 'valid_loss'. Value
+                                should be one of the metric that is displayed in
+                                the training table. Use `{model_name}.available_metrics`
+                                to list the available metrics to set here.
         =====================   ===========================================
         """
         self._check_requisites()
@@ -668,7 +750,7 @@ class ArcGISModel(object):
                     else metric.__qualname__
                     for metric in self.learn.metrics
                 ]
-        ):
+        ) and not getattr(self, "_is_edge_detection", False):
             self.learn.metrics.extend([dice])
         if arcgis.env.verbose:
             logger.info('Fitting the model.')
@@ -678,14 +760,19 @@ class ArcGISModel(object):
 
         callbacks = kwargs['callbacks'] if 'callbacks' in kwargs.keys() else []
         kwargs.pop('callbacks', None)
+        monitored_names = self.available_metrics
+        if monitor not in monitored_names:
+            raise Exception(f"`monitor` must be set to one from {monitored_names}")
+        self.monitor = monitor
         if early_stopping:
-            callbacks.append(EarlyStoppingCallback(learn=self.learn, monitor='valid_loss', min_delta=0.01, patience=5))
+            callbacks.append(EarlyStoppingCallback(learn=self.learn, monitor=monitor, min_delta=0.001, patience=5))
         if checkpoint:
             from datetime import datetime
             now = datetime.now()
-            save_callback_params = kwargs.get("save_callback_params", {"monitor": "valid_loss", "every": "improvement"})
-            # callbacks.append(SaveModelCallback(self, monitor='valid_loss', every='improvement',
-            #                                   name=now.strftime("checkpoint_%Y-%m-%d_%H-%M-%S")))
+            if checkpoint != True and checkpoint != "all":
+                raise Exception("Checkpoint can only be set to a boolean or 'all'")
+            every = "improvement" if checkpoint is True else "epoch"
+            save_callback_params = kwargs.get("save_callback_params", {"monitor": monitor, "every": every})
             callbacks.append(SaveModelCallback(self,
                                                name=now.strftime("checkpoint_%Y-%m-%d_%H-%M-%S"),
                                                **save_callback_params))
@@ -693,17 +780,20 @@ class ArcGISModel(object):
         # If tensorboardx is installed write a log with name as timestamp
         if tensorboard and HAS_TENSORBOARDX:
             training_id = time.strftime("log_%Y-%m-%d_%H-%M-%S")
-            log_path = Path(os.path.dirname(self._data.path)) / 'training_log'
+            log_path = Path(self._data.path) / 'training_log'
             abs_path = os.path.abspath(log_path)
             training_id = type(self).__name__ + "_" + training_id
             callbacks.append(
                 partial(ArcGISTBCallback, base_dir=log_path, name=training_id, arcgis_model=self)(learn=self.learn))
             hostname = socket.gethostname()
-            print("Monitor training on Tensorboard using the following command: 'tensorboard --host={} --logdir=\"{}\"'".format(hostname, abs_path))
+            print(
+                "Monitor training on Tensorboard using the following command: 'tensorboard --host={} --logdir=\"{}\"'".format(
+                    hostname, abs_path))
         # Send out a warning if tensorboardX is not installed
         elif tensorboard:
             warn("Install tensorboardX 2.1 'pip install tensorboardx==2.1' to write training log")
 
+        self._fit_callbacks = callbacks
         if one_cycle:
             self.learn.fit_one_cycle(epochs, lr, callbacks=callbacks, **kwargs)
         else:
@@ -760,10 +850,28 @@ class ArcGISModel(object):
                 backbone = self._orig_backbone.__name__
 
         _emd_template = self._get_emd_params(save_inference_file)
+        if getattr(self, '_data', None) is not None:
+            _emd_template['MinCellSize'] = getattr(self._data, '_emd', {}).get('MinCellSize', None)
+            _emd_template['MaxCellSize'] = getattr(self._data, '_emd', {}).get('MaxCellSize', None)
 
         _emd_template["SupportsVariableTileSize"] = _emd_template.get("SupportsVariableTileSize", False)
         _emd_template["ArcGISLearnVersion"] = ArcGISLearnVersion
 
+        if getattr(self, '_fit_callbacks', None) is not None:
+            checkpoint_callback = [c for c in self._fit_callbacks if isinstance(c, SaveModelCallback)]
+            if checkpoint_callback != []:
+                checkpoint_callback = checkpoint_callback[0]
+                key = getattr(self, 'monitor', 'valid_loss')
+                if checkpoint_callback.every == 'improvement':
+                    val = checkpoint_callback.best
+                else:
+                    val = checkpoint_callback.current
+                if isinstance(val, torch.Tensor):
+                    val = val.cpu().item()
+                else:
+                    val = float(val)
+                _emd_template[f'monitored_{key}'] = val
+
         if isinstance(self._learning_rate, slice):
             _emd_lr = slice('{0:1.4e}'.format(self._learning_rate.start), '{0:1.4e}'.format(self._learning_rate.stop))
         elif self._learning_rate is not None:
@@ -926,11 +1034,17 @@ class ArcGISModel(object):
             <p><b>Score:</b> {emd_template.get('score')}</p>
         """
 
-        if emd_template.get('psnr_metric'):
+        if emd_template.get('PSNR'):
             model_analysis = f"""
-            <p><b>PSNR Metric:</b> {emd_template.get('psnr_metric')}</p>
-            <p><b>SSIM Metric:</b> {emd_template.get('ssim_metric')}</p>
+            <p><b>PSNR Metric:</b> {emd_template.get('PSNR')}</p>
+            <p><b>SSIM Metric:</b> {emd_template.get('SSIM')}</p>
         """
+            # FID is supported for RGB only
+            if emd_template.get('FID'):
+                model_analysis = model_analysis + f"""
+                <p><b>FID Metric:</b> {emd_template.get('FID')}</p>
+                """
+
         if emd_template.get('per_class_metrics'):
             html_table = pd.read_json(emd_template.get('per_class_metrics')).to_html()
             model_analysis = f"""
@@ -984,6 +1098,7 @@ class ArcGISModel(object):
         post_processed = kwargs.get('post_processed', True)  # True, False
         quantized = kwargs.get('quantized', False)  # True, False
         temp = self.learn.path
+        temp1 = self.learn.model_dir
         if '\\' in name_or_path or '/' in name_or_path:
             path = Path(name_or_path)
             name = path.parts[-1]
@@ -994,7 +1109,7 @@ class ArcGISModel(object):
                 os.makedirs(self.learn.path)
         else:
             # fixing fastai bug
-            self.learn.path = self.learn.path.parent
+            # self.learn.path = self.learn.path.parent
             self.learn.model_dir = Path(self.learn.model_dir) / name_or_path
             if not os.path.exists(self.learn.path / self.learn.model_dir):
                 os.makedirs(self.learn.path / self.learn.model_dir)
@@ -1029,7 +1144,7 @@ class ArcGISModel(object):
         finally:
 
             self.learn.path = temp
-            self.learn.model_dir = 'models'
+            self.learn.model_dir = temp1
 
         _emd_template = self._create_emd_template(saved_path.with_suffix('.pth'), compute_metrics, save_inference_file)
 
@@ -1042,6 +1157,75 @@ class ArcGISModel(object):
             self._create_tfonnx_emd_template(_emd_template, saved_path.with_suffix('.onnx'), batch_size)
             os.remove(saved_path.with_suffix('.pth'))
 
+        if framework.lower() == "torchscript":
+            from ._siammask_utils import Custom
+            from ._siammask_utils import load_pretrain
+            siammask = Custom(anchors=self.anchors)
+            if '\\' in name_or_path or '/' in name_or_path:
+                models_path = os.path.join(name_or_path)
+            else:
+                models_path = os.path.join(self.learn.path, self.learn.model_dir, name)
+            if not os.path.exists(models_path):
+                os.makedirs(models_path)
+
+            siammask = load_pretrain(siammask, os.path.join(models_path, name + ".pth"))
+            outdir = os.path.join(models_path, 'torch_scripts')
+            if not os.path.isdir(outdir):
+                os.mkdir(outdir)
+
+            scripted_feature_extractor = torch.jit.script(siammask.features.features)
+            scripted_feature_extractor.save(os.path.join(outdir, 'feature_extractor.pt'))
+
+            scripted_feature_downsampler = torch.jit.script(siammask.features.downsample)
+            scripted_feature_downsampler.save(os.path.join(outdir, 'feature_downsampler.pt'))
+
+            scripted_rpn_model = torch.jit.script(siammask.rpn_model)
+            scripted_rpn_model.save(os.path.join(outdir, 'rpn_model.pt'))
+
+            scripted_mask_conv_kernel = torch.jit.script(siammask.mask_model.mask.conv_kernel)
+            scripted_mask_conv_kernel.save(os.path.join(outdir, 'mask_conv_kernel.pt'))
+
+            scripted_mask_conv_search = torch.jit.script(siammask.mask_model.mask.conv_search)
+            scripted_mask_conv_search.save(os.path.join(outdir, 'mask_conv_search.pt'))
+
+            scripted_mask_depthwise_conv = torch.jit.script(siammask.mask_model.mask.conv2d_dw_group)
+            scripted_mask_depthwise_conv.save(os.path.join(outdir, 'mask_depthwise_conv.pt'))
+
+            scripted_refine_model = torch.jit.script(siammask.refine_model)
+            scripted_refine_model.save(os.path.join(outdir, 'refine_model.pt'))
+            temp_emd_template = _emd_template.copy()
+            temp_emd_template["ModelFile"] = "."
+            temp_emd_template["ModelFiles"] = [
+                "feature_extractor.pt",
+                "feature_downsampler.pt",
+                "rpn_model.pt",
+                "mask_conv_kernel.pt",
+                "mask_conv_search.pt",
+                "mask_depthwise_conv.pt",
+                "refine_model.pt"
+            ]
+
+            if os.path.exists(os.path.join(outdir, name + ".emd")):
+                os.remove(os.path.join(outdir, name + ".emd"))
+
+            import zipfile
+            dlpk_Name = os.path.join(outdir, name + '.dlpk')
+            if os.path.exists(dlpk_Name):
+                os.remove(dlpk_Name)
+
+            out_file = open(os.path.join(outdir, name + ".emd"), "w")
+            json.dump(temp_emd_template, out_file, indent=4)
+            out_file.close()
+            dlpk_Name = os.path.join(outdir, name + '.dlpk')
+            f = zipfile.ZipFile(dlpk_Name, 'w')
+            cwd = os.getcwd()
+            os.chdir(outdir)
+            for files in temp_emd_template["ModelFiles"]:
+                f.write(files)
+
+            f.write(name + ".emd")
+            f.close()
+            os.chdir(cwd)
 
         if _emd_template.get('InferenceFunction', False):
             if _emd_template['ModelType'] not in ["ObjectDetection", "ImageClassification", "InstanceDetection", \
@@ -1056,19 +1240,25 @@ class ArcGISModel(object):
             if not save_inference_file:
                 inference_file = _emd_template['InferenceFunction']
                 if "[Functions]" not in inference_file:
-                    _emd_template["InferenceFunction"] = "[Functions]System\\DeepLearning\\ArcGISLearn\\" + _emd_template["InferenceFunction"]
-
-
+                    _emd_template["InferenceFunction"] = "[Functions]System\\DeepLearning\\ArcGISLearn\\" + \
+                                                         _emd_template["InferenceFunction"]
 
         ArcGISModel._write_emd(_emd_template, saved_path.with_suffix('.emd'))
         zip_name = saved_path.stem
 
         if save_html:
             try:
+                if _IS_ARCGISPRONOTEBOOK:
+                    from IPython import get_ipython
+                    get_ipython().run_line_magic('matplotlib', 'auto')
                 self._save_model_characteristics(saved_path.parent.absolute() / model_characteristics_folder)
                 ArcGISModel._create_html(saved_path)
             except:
                 pass
+            finally:
+                if _IS_ARCGISPRONOTEBOOK:
+                    from IPython import get_ipython
+                    get_ipython().run_line_magic('matplotlib', 'inline')
 
         if _emd_template.get('ModelConfigurationFile', False):
             with open(saved_path.parent / _emd_template['ModelConfigurationFile'], 'w') as f:
@@ -1236,8 +1426,11 @@ class ArcGISModel(object):
                                 model. (Only supported by ``SingleShotDetector``, currently.)
                                 If framework used is ``TF-ONNX``, ``batch_size`` can be
                                 passed as an optional keyword argument.
+                                If framework used is `torchscript`, pytorch(.pt) files will
+                                get generated for the model inside 'torch_scripts'. (Only
+                                supported by `SiamMask`)
 
-                                Framework choice: 'PyTorch' and 'TF-ONNX'
+                                Framework choice: 'PyTorch', 'TF-ONNX' and 'torchscript'
         ---------------------   -------------------------------------------
         publish                 Optional boolean. Publishes the DLPK as an item.
         ---------------------   -------------------------------------------
@@ -1290,7 +1483,7 @@ class ArcGISModel(object):
             self.learn.model_dir = ''
         else:
             # fixing fastai bug
-            self.learn.path = self.learn.path.parent
+            # self.learn.path = self.learn.path.parent
             self.learn.model_dir = Path(self.learn.model_dir) / name_or_path
             name = name_or_path
 
@@ -1301,4 +1494,4 @@ class ArcGISModel(object):
         finally:
             # undoing changes to self.learn.path
             self.learn.path = temp
-            self.learn.model_dir = 'models'
+            self.learn.model_dir = 'models'
\ No newline at end of file
