@@ -39,6 +39,7 @@ try:
     import PIL.ExifTags
     from torch.nn import Module as NnModule
     from .._utils.common import get_multispectral_data_params_from_emd, _get_emd_path, image_batch_stretcher
+    from .._utils.env import _IS_ARCGISPRONOTEBOOK
     from matplotlib import pyplot as plt
     import copy
     HAS_FASTAI = True
@@ -155,7 +156,11 @@ class FeatureClassifier(ArcGISModel):
             if mixup:
                 # For mixup to work with multilabel call it with parameter stack_y=False
                 stack_y = getattr(data, '_dataset_type', "Labeled_Tiles") == 'Labeled_Tiles'
-                self.learn = self.learn.mixup(stack_y=stack_y)
+                if ((getattr(data, '_dataset_type', "Labeled_Tiles") == 'Labeled_Tiles') or
+                        (getattr(data, '_dataset_type', "Labeled_Tiles") == 'MultiLabeled_Tiles')):
+                    self.learn = self.learn.mixup(stack_y=stack_y)
+                else:
+                    self.learn = self.learn.mixup()
 
             self.learn.model = self.learn.model.to(self._device)
 
@@ -185,7 +190,7 @@ class FeatureClassifier(ArcGISModel):
     
     @staticmethod
     def _supported_datasets():
-        return ['Labeled_Tiles', 'MultiLabeled_Tiles']  
+        return ['Labeled_Tiles', 'MultiLabeled_Tiles', 'Imagenet']
 
     def show_results(self, rows=5, **kwargs):
         """
@@ -197,6 +202,8 @@ class FeatureClassifier(ArcGISModel):
             rows = math.floor(math.sqrt(len(self._data.valid_ds)))
 
         self.learn.show_results(rows=rows, **kwargs)
+        if _IS_ARCGISPRONOTEBOOK:
+            plt.show()
    
     def _show_results_multispectral(self, rows=5, **kwargs):
         from .._utils.image_classification import IC_show_results
@@ -364,6 +371,7 @@ class FeatureClassifier(ArcGISModel):
         Plots a confusion matrix of the model predictions to evaluate accuracy
         kwargs: 'thresh' - confidence score threshold for multilabel predictions, defaults to 0.5
         """
+        self._check_requisites()
         if self._data._dataset_type == 'MultiLabeled_Tiles':
             # Get x, y from validation dataset
             data_loader = self._data.valid_dl
@@ -423,8 +431,16 @@ class FeatureClassifier(ArcGISModel):
         else:
             self._check_requisites()
             learn_temp = copy.copy(self.learn)
+            
+            # Reassigning the function from vision.learner because fastai sets it from tabular.learner
+            from fastai.vision.learner import _cl_int_from_learner
+            ClassificationInterpretation.from_learner = _cl_int_from_learner
             interp = ClassificationInterpretation.from_learner(learn_temp)
-            interp.plot_confusion_matrix()
+            
+            nrows = self._data.c
+            # figsize range: 4 <= (no. of classes + 15)/4 <=20
+            fs = min(max(4, (nrows+15)/4), 20)
+            interp.plot_confusion_matrix(figsize=(fs,fs))
 
     def plot_hard_examples(self, num_examples):
         """
