@@ -1,4 +1,3 @@
-
 code = """
 import json
 import sys, os, importlib
@@ -262,7 +261,7 @@ class ArcGISObjectDetector:
                     'rings': rings
                 }
             })   
-            
+
         return {'output_vectors': json.dumps(features)}
 
 """
@@ -594,7 +593,7 @@ class ArcGISObjectClassifier:
 
         return {'output_vectors': json.dumps(features)}
 """
-entity_recognizer_placeholder= """
+entity_recognizer_placeholder = """
 print('not implemented')
 """
 
@@ -707,7 +706,7 @@ attribute_table = {
     'features': []
 }
 
- 
+
 
 class ArcGISImageClassifier:
     def __init__(self):
@@ -801,7 +800,11 @@ class ArcGISImageClassifier:
     def updateRasterInfo(self, **kwargs):
         kwargs['output_info']['bandCount'] = 1
         #todo: type is determined by the value range of classes in the json file
-        kwargs['output_info']['pixelType'] = 'i4'
+        prob_raster = getattr(self.child_image_classifier,'probability_raster',False)
+        if prob_raster:
+            kwargs['output_info']['pixelType'] = 'f4' # To ensure that output pixels are in prob range 0 to 1
+        else:
+            kwargs['output_info']['pixelType'] = 'i4'
         class_info = self.json_info['Classes']
         attribute_table['features'] = []
         for i, c in enumerate(class_info):
@@ -824,18 +827,22 @@ class ArcGISImageClassifier:
 
     def updatePixels(self, tlc, shape, props, **pixelBlocks):
         # set pixel values in invalid areas to 0
-           
+
         raster_mask = pixelBlocks['raster_mask']
         raster_pixels = pixelBlocks['raster_pixels']
         raster_pixels[np.where(raster_mask == 0)] = 0
         pixelBlocks['raster_pixels'] = raster_pixels
 
-        xx = self.child_image_classifier.updatePixels(tlc, shape, props, **pixelBlocks).astype(props['pixelType'], copy=False)        
-        tytx = getattr(self.child_image_classifier, 'tytx', self.json_info['ImageHeight'])
-        chunks, num_rows, num_cols =  chunk_it(xx.transpose(1, 2, 0), tytx)# self.json_info['ImageHeight'])  # ImageHeight = ImageWidth
-        xx = patch_chips(crop_flatten(chunks, self.child_image_classifier.padding), num_rows, num_cols)
-        xx = xx.transpose(2, 0, 1)
-        pixelBlocks['output_pixels'] = xx
+        if self.json_info['ModelName'] == 'MultiTaskRoadExtractor':
+            xx = self.child_image_classifier.detectRoads(tlc, shape, props, **pixelBlocks).astype(props['pixelType'], copy=False)   
+            pixelBlocks['output_pixels'] = xx
+        else:
+            xx = self.child_image_classifier.updatePixels(tlc, shape, props, **pixelBlocks).astype(props['pixelType'], copy=False)   
+            tytx = getattr(self.child_image_classifier, 'tytx', self.json_info['ImageHeight'])
+            chunks, num_rows, num_cols =  chunk_it(xx.transpose(1, 2, 0), tytx)# self.json_info['ImageHeight'])  # ImageHeight = ImageWidth
+            xx = patch_chips(crop_flatten(chunks, self.child_image_classifier.padding), num_rows, num_cols)
+            xx = xx.transpose(2, 0, 1)
+            pixelBlocks['output_pixels'] = xx
 
         return pixelBlocks
 """
@@ -1013,7 +1020,7 @@ class ArcGISInstanceDetector:
         self.child_instance_detector = ChildInstanceDetector()
         self.child_instance_detector.initialize(model, model_as_file)
 
-        
+
     def getParameterInfo(self):       
         required_parameters = [
             {
@@ -1056,7 +1063,7 @@ class ArcGISInstanceDetector:
         return GeometryType.Polygon        
 
     def vectorize(self, **pixelBlocks):
-           
+
         raster_mask = pixelBlocks['raster_mask']
         raster_pixels = pixelBlocks['raster_pixels']
         raster_pixels[np.where(raster_mask == 0)] = 0
@@ -1176,7 +1183,7 @@ def patch_chips(imgs, n_rows, n_cols):
     for i in range(n_rows):
         h_stacks.append(np.hstack(imgs[i*n_cols:n_cols*(i+1) ]))
     return np.vstack(h_stacks)
- 
+
 
 class ArcGISSuperResolution:
     def __init__(self):
@@ -1274,7 +1281,7 @@ class ArcGISSuperResolution:
 
     def updatePixels(self, tlc, shape, props, **pixelBlocks):
         # set pixel values in invalid areas to 0
-           
+
         #raster_mask = pixelBlocks['raster_mask']
         raster_pixels = pixelBlocks['raster_pixels']
         #raster_pixels[np.where(raster_mask == 0)] = 0
@@ -1344,7 +1351,7 @@ def patch_chips(imgs, n_rows, n_cols):
     for i in range(n_rows):
         h_stacks.append(np.hstack(imgs[i*n_cols:n_cols*(i+1) ]))
     return np.vstack(h_stacks)
- 
+
 class ArcGISImageTranslation:
     def __init__(self):
         self.name = 'Image Classifier'
@@ -1431,7 +1438,7 @@ class ArcGISImageTranslation:
         return kwargs
     def updatePixels(self, tlc, shape, props, **pixelBlocks):
         # set pixel values in invalid areas to 0
-           
+
         #raster_mask = pixelBlocks['raster_mask']
         raster_pixels = pixelBlocks['raster_pixels']
         #raster_pixels[np.where(raster_mask == 0)] = 0
