@@ -1,6 +1,6 @@
 from pathlib import Path
 import json
-from ._arcgis_model import _EmptyData, _change_tail, ArcGISModel
+from ._arcgis_model import _EmptyData, _change_tail, ArcGISModel, _get_device
 from ._codetemplate import code, image_classifier_prf
 import warnings
 import arcgis
@@ -36,6 +36,8 @@ try:
     from .._image_utils import _get_image_chips, _get_transformed_predictions, _draw_predictions, _exclude_detection
     from .._video_utils import VideoUtils
     import inspect
+    from .._utils.env import _IS_ARCGISPRONOTEBOOK
+    from matplotlib import pyplot as plt
     HAS_FASTAI = True
     
 except Exception as e:
@@ -247,7 +249,7 @@ class ModelExtension(ArcGISModel):
                 with warnings.catch_warnings():
                     warnings.simplefilter("ignore", UserWarning)
                     sd = ImageList([], path=emd_path.parent.parent).split_by_idx([])
-                    data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch().normalize(imagenet_stats)
+                    data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch(device=_get_device()).normalize(imagenet_stats)
                 # Add 1 for background class
                 data.c += 1
             else:
@@ -417,6 +419,8 @@ class ModelExtension(ArcGISModel):
             ys = [ds.y.reconstruct(grab_idx(y, i)) for i in range(n_items)]
             zs = [ds.y.reconstruct(z) for z in preds]
         ds.x.show_xyzs(xs, ys, zs, **kwargs)
+        if _IS_ARCGISPRONOTEBOOK:
+            plt.show()
 
     def _predict_learn_modified(self, item, **kwargs):
         "Return predicted class, label and probabilities for `item`."
