@@ -21,6 +21,7 @@ HAS_ARCPY = True
 # so that relevant exception can be raised
 try:
     import numpy as np
+    import torch
     from fastai.vision.learner import create_body
     from torch import optim
     from torchvision import models
@@ -30,16 +31,21 @@ try:
         miou,
         road_orient_loss,
     )
+    from arcgis.learn.models._linknet_utils import compute_miou
     from arcgis.learn.models._hourglass_utils import StackHourglassMultiTaskModel
     from ._road_mtl_learner import MultiTaskRoadLearner
+    from ._unet_utils import show_results_multispectral
     from arcgis.learn.models._arcgis_model import _resnet_family, _EmptyData
     from arcgis.learn._utils.common import (
         get_multispectral_data_params_from_emd ,_get_emd_path
     )
     from arcgis.learn._utils.segmentation_loss_functions import dice
     from arcgis.learn.models._arcgis_model import _device_check
+    from ._arcgis_model import _set_multigpu_callback
     from .._data_utils._pixel_classifier_data import ClassifiedTilesData
     from .._data_utils._road_orient_data import RoadOrientation
+    from .._utils.env import _IS_ARCGISPRONOTEBOOK
+    from matplotlib import pyplot as plt
 except Exception as e:
     import_exception = "\n".join(
         traceback.format_exception(type(e), e, e.__traceback__)
@@ -64,9 +70,8 @@ def safe_json(data):
 
 class MultiTaskRoadExtractor(ArcGISModel):
     """
-    Creates a Multi-Task Learning model for binary segmentation. Supports 8 bit
-    RGB imagery and offers experimental support for 16 bit RGB imagery.
-    Does not support Multispectral imagery yet.
+    Creates a Multi-Task Learning model for binary segmentation of roads. Supports RGB
+    and Multispectral Imagery.
     Implementation based on https://doi.org/10.1109/CVPR.2019.01063 .
 
     =====================   ===========================================
@@ -169,10 +174,12 @@ class MultiTaskRoadExtractor(ArcGISModel):
 
                 kwargs['orient_bin_size'] = bin_size #To ensure the data is consistent across multiple runs
 
+
             self._orient_data=self._get_road_orient_data(data,**kwargs)
-            if len(data.classes)>2:
+            if len(data.classes) > 2:
                 raise Exception(
-                    "Found multi-labels in the data, This is a binary segmentation model and hence please export the data with binary labels."  # noqa
+                    "Found multi-labels in the data, This is a binary segmentation model and hence please export the data with binary labels."
+                    # noqa
                 )
             self._data.classes = self._orient_data.classes
             class_key = self._orient_data.classes[0] if isinstance(self._orient_data.classes[0], int) else \
@@ -202,7 +209,7 @@ class MultiTaskRoadExtractor(ArcGISModel):
                                                                                           '_min_max_scaler') else None
                 self._orient_data._min_max_scaler_tfm = self._data._min_max_scaler_tfm if hasattr(data,
                                                                                                   '_min_max_scaler_tfm') else None
-                self._orient_data._multispectral_color_array = self._data._multispectral_color_array if hasattr(data,
+                self._orient_data._multispectral_color_array = self._data._multispectral_color_array*255 if hasattr(data,
                                                                                                                 '_multispectral_color_array') else None
                 self._orient_data._multispectral_color_mapping = self._data._multispectral_color_mapping if hasattr(
                     data, '_multispectral_color_mapping') else None
@@ -222,6 +229,11 @@ class MultiTaskRoadExtractor(ArcGISModel):
                 self._orient_data._do_normalize = self._data._do_normalize if hasattr(data, '_do_normalize') else None
                 self._orient_data.train_ds.tfms = self._data.train_ds.tfms if hasattr(data.train_ds, 'tfms') else None
                 self._orient_data.valid_ds.tfms = self._data.valid_ds.tfms if hasattr(data.valid_ds, 'tfms') else None
+                if self._orient_data._do_normalize:
+                    self._orient_data = self._orient_data.normalize(stats=(self._orient_data._scaled_mean_values, self._orient_data._scaled_std_values), do_x=True,
+                                          do_y=False)
+                #self._data = self._orient_data
+                self.show_results = self._show_results_multispectral
         else:
             self._orient_data =self._data # Else use the data attributes obtained from emd
             #if self._orient_data._do_normalize:
@@ -310,10 +322,12 @@ class MultiTaskRoadExtractor(ArcGISModel):
             self._model,
             loss_func=self._loss_f,
             metrics=[pixel_accuracy, road_iou, dice_coeff],
-            **learner_kwargs,
+            **learner_kwargs
         )
-        self.learn.path = self._data.path
+        if hasattr(self._data,'path'):
+            self.learn.path=self._data.path
         self.learn.model = self.learn.model.to(self._device)
+        _set_multigpu_callback(self)
         if pretrained_path is not None:
             super().load(str(pretrained_path))
         self._arcgis_init_callback()  # make first conv weights learnable
@@ -321,6 +335,36 @@ class MultiTaskRoadExtractor(ArcGISModel):
     def __str__(self):
         return self.__repr__()
 
+    def mIOU(self, mean=False, show_progress=True):
+
+        """
+        Computes mean IOU on the validation set for each class.
+
+        =====================   ===========================================
+        **Argument**            **Description**
+        ---------------------   -------------------------------------------
+        mean                    Optional bool. If False returns class-wise
+                                mean IOU, otherwise returns mean iou of all
+                                classes combined.
+        ---------------------   -------------------------------------------
+        show_progress           Optional bool. Displays the prgress bar if
+                                True.
+        =====================   ===========================================
+
+        :returns: `dict` if mean is False otherwise `float`
+        """
+        #self._check_requisites()
+        num_classes = torch.arange(self._orient_data.c)
+        miou = compute_miou(self, self._orient_data.valid_dl, mean, num_classes, show_progress, self._ignore_mapped_class)
+        if mean:
+            miou = [miou[i] for i in range(len(miou)) if i not in self._ignore_mapped_class]
+            return np.mean(miou)
+        if self._ignore_mapped_class == []:
+            return dict(zip(['0'] + self._orient_data.classes[1:], miou))
+        else:
+            class_values = [0] + list(self._orient_data.class_mapping.keys())
+            return {class_values[i]: miou[i] for i in range(len(miou)) if i not in self._ignore_mapped_class}
+
     def __repr__(self):
         return "<%s>" % (type(self).__name__)
 
@@ -332,7 +376,7 @@ class MultiTaskRoadExtractor(ArcGISModel):
             path=self._data.orig_path, class_mapping={}, chip_size=self._data.chip_size, val_split_pct=self._data._val_split_pct,
             batch_size=self._data.batch_size, transforms=None, seed=42,
             dataset_type=self._data.dataset_type, resize_to=self._data.resize_to, **kwargs)
-        road_orient_obj = RoadOrientation(road_data_obj, **kwargs)
+        road_orient_obj = RoadOrientation(self._data,road_data_obj, **kwargs)
         orient_data = road_orient_obj.get_databunch(data,**kwargs)
         return orient_data
 
@@ -377,16 +421,10 @@ class MultiTaskRoadExtractor(ArcGISModel):
         return ['Classified_Tiles']
 
     def fit(self, epochs=10, lr=None, **kwargs):
-        save_callback_params = {
-            "monitor": "miou",
-            "every": "improvement",
-            "load_best_at_end": False,
-        }
-        save_callback_params.update(kwargs.get("save_callback_params", {}))
-        kwargs.update(save_callback_params=save_callback_params)
         if isinstance(lr, slice):
             lr = lr.stop
-        super().fit(epochs, lr=lr, **kwargs)
+        # setting monitor because earlystopping also uses this value.
+        super().fit(epochs, lr=lr, monitor=kwargs.pop('monitor', "miou"), **kwargs)
 
     def _get_emd_params(self,save_inference_file):
         _emd_template = {}
@@ -515,9 +553,23 @@ class MultiTaskRoadExtractor(ArcGISModel):
         self._check_requisites()
         self.return_fig = kwargs.get("return_fig", False)
         fig=self.learn.show_results(rows=rows, **kwargs)
+        if _IS_ARCGISPRONOTEBOOK:
+            plt.show()
         if self.return_fig:
             return fig
 
+    def _show_results_multispectral(self, rows=5, alpha=0.7, **kwargs):  # parameters adjusted in kwargs
+        return_fig = kwargs.get('return_fig', False)
+        ret_val = show_results_multispectral(
+            self,
+            nrows=rows,
+            alpha=alpha,
+            **kwargs
+        )
+        if return_fig:
+            fig, ax = ret_val
+            return fig
+
     @property
     def _model_metrics(self):
         acc, mean_iou = self._get_model_metrics()
