@@ -3,9 +3,24 @@ from .._utils.env import raise_fastai_import_error
 
 import_exception = None
 try:
+    from .._data import _raise_fastai_import_error 
     from ._arcgis_model import ArcGISModel, SaveModelCallback, _set_multigpu_callback
-    from ._pointcnn_utils import PointCNNSeg, SamplePointsCallback, CrossEntropyPC, accuracy, accuracy_non_zero, AverageMetric
-    from .._utils.pointcloud_data import get_device, inference_las, show_results, compute_precision_recall, predict_h5, show_results_tool
+    from ._pointcnn_utils import (PointCNNSeg,
+                                  SamplePointsCallback, 
+                                  CrossEntropyPC, 
+                                  accuracy, 
+                                  CalculateClassificationReport,
+                                  accuracy_non_zero, 
+                                  AverageMetric,
+                                  precision,
+                                  recall,
+                                  f1)
+    from .._utils.pointcloud_data import (get_device, 
+                                          inference_las, 
+                                          show_results, 
+                                          compute_precision_recall, 
+                                          predict_h5, 
+                                          show_results_tool)
     from ._unet_utils import is_no_color
     from fastai.basic_train import Learner
     import torch
@@ -81,10 +96,16 @@ class PointCNN(ArcGISModel):
         self._backbone = None
         self.sample_point_num = kwargs.get('sample_point_num', data.max_point)
         self.learn = Learner(data,
-                PointCNNSeg(self.sample_point_num, data.c, data.extra_dim, kwargs.get('encoder_params', None), kwargs.get('dropout', None)),
-                loss_func=CrossEntropyPC(data.c),
-                metrics=[AverageMetric(accuracy)],
-                callback_fns=[partial(SamplePointsCallback, sample_point_num=self.sample_point_num)])
+                             PointCNNSeg(self.sample_point_num, data.c, data.extra_dim, kwargs.get('encoder_params', None), kwargs.get('dropout', None)),
+                             loss_func=CrossEntropyPC(data.c),
+                             metrics=[AverageMetric(accuracy),
+                                      AverageMetric(precision), 
+                                      AverageMetric(recall), 
+                                      AverageMetric(f1)
+                                     ],
+                             callback_fns=[partial(SamplePointsCallback, sample_point_num=self.sample_point_num),
+                                           CalculateClassificationReport,
+                                          ])
         self.encoder_params = self.learn.model.encoder_params
 
         self.learn.model = self.learn.model.to(self._device)
@@ -112,6 +133,8 @@ class PointCNN(ArcGISModel):
 
         :returns: `PointCNN` Object
         """      
+        if not HAS_FASTAI:
+            _raise_fastai_import_error(import_exception=import_exception)        
 
         emd_path = _get_emd_path(emd_path)
         with open(emd_path) as f:
@@ -135,22 +158,30 @@ class PointCNN(ArcGISModel):
             data.emd = emd
             for key, value in emd['DataAttributes'].items():
                 setattr(data, key, value)
+
             ## backward compatibility.
             if not hasattr(data, 'class_mapping'):
                 data.class_mapping = class_mapping
+                if not hasattr(data, 'class2idx'):
+                    data.class2idx = class_mapping   
             if not hasattr(data, 'color_mapping'):
                 data.color_mapping = color_mapping
+            if not hasattr(data, 'class2idx'):
+                data.class2idx = data.class_mapping 
             if not hasattr(data, 'classes'):
-                data.classes = list(class_mapping.values())
+                data.classes = list(data.class2idx.values())                           
+
+
+            data.class2idx = {int(k): int(v) for k, v in data.class2idx.items()}
+            if hasattr(data, 'idx2class'):
+                data.idx2class = {int(k): int(v) for k, v in data.idx2class.items()}
 
-            data.class_mapping = {int(k): int(v) for k, v in data.class_mapping.items()}
             data.color_mapping = {int(k): v for k, v in data.color_mapping.items()}
 
-
             ## Below are the lines to make save function work
             data.chip_size = None
             data._image_space_used = None
-            data.dataset_type = 'PointCloud'                 
+            data.dataset_type = 'PointCloud'              
 
         return cls(data, **model_params, pretrained_path=str(model_file))
 
@@ -164,7 +195,8 @@ class PointCNN(ArcGISModel):
 
         """
         Train the model for the specified number of epochs and using the
-        specified learning rates
+        specified learning rates. The precision, recall and f1 scores
+        shown in the training table are macro averaged over all classes.
         
         =====================   ===========================================
         **Argument**            **Description**
@@ -172,23 +204,29 @@ class PointCNN(ArcGISModel):
         epochs                  Required integer. Number of cycles of training
                                 on the data. Increase it if underfitting.
         ---------------------   -------------------------------------------
-        lr                      Optional float. Learning rate to be used
-                                for training the model. If ``lr=None``, an
-                                optimal learning rate is automatically deduced
+        lr                      Optional float or slice of floats. Learning rate
+                                to be used for training the model. If ``lr=None``,
+                                an optimal learning rate is automatically deduced
                                 for training the model.
         ---------------------   -------------------------------------------
         one_cycle               Optional boolean. Parameter to select 1cycle
-                                learning rate schedule. If set to `False` no 
-                                learning rate schedule is used.       
+                                learning rate schedule. If set to `False` no
+                                learning rate schedule is used.
         ---------------------   -------------------------------------------
         early_stopping          Optional boolean. Parameter to add early stopping.
-                                If set to 'True' training will stop if validation
-                                loss stops improving for 5 epochs.        
+                                If set to 'True' training will stop if parameter
+                                `monitor` value stops improving for 5 epochs.
+                                A minimum difference of 0.001 is required for
+                                it to be considered an improvement.
         ---------------------   -------------------------------------------
-        checkpoint              Optional boolean. Parameter to save the best model
-                                during training. If set to `True` the best model 
-                                based on validation loss will be saved during 
-                                training.
+        checkpoint              Optional boolean or string.
+                                Parameter to save checkpoint during training.
+                                If set to `True` the best model
+                                based on `monitor` will be saved during
+                                training. If set to 'all', all checkpoints
+                                are saved. If set to False, checkpointing will
+                                be off. Setting this parameter loads the best
+                                model at the end of training.
         ---------------------   -------------------------------------------
         tensorboard             Optional boolean. Parameter to write the training log.
                                 If set to 'True' the log will be saved at
@@ -197,6 +235,13 @@ class PointCNN(ArcGISModel):
 
                                 The default value is 'False'.
                                 **Note - Not applicable for Text Models
+        ---------------------   -------------------------------------------
+        monitor                 Optional string. Parameter specifies
+                                which metric to monitor while checkpointing
+                                and early stopping. Defaults to 'valid_loss'. Value
+                                should be one of the metric that is displayed in
+                                the training table. Use `{model_name}.available_metrics`
+                                to list the available metrics to set here.
         =====================   ===========================================
 
         **kwargs**
@@ -265,17 +310,29 @@ class PointCNN(ArcGISModel):
         _emd_template['DataAttributes']['max_point'] = self._data.max_point
         _emd_template['DataAttributes']['extra_features'] = self._data.extra_features
         _emd_template['DataAttributes']['extra_dim'] = self._data.extra_dim
-        _emd_template['DataAttributes']['class_mapping'] = self._data.class_mapping
+        _emd_template['DataAttributes']['class2idx'] = self._data.class2idx
         _emd_template['DataAttributes']['color_mapping'] = self._data.color_mapping
         _emd_template['DataAttributes']['remap'] = self._data.remap
         _emd_template['DataAttributes']['classes'] = self._data.classes
+        _emd_template['DataAttributes']['subset_classes'] = self._data.subset_classes
+        _emd_template['DataAttributes']['class_mapping'] = self._data.class_mapping
+        _emd_template['DataAttributes']['remap_dict'] = self._data.remap_dict
+        _emd_template['DataAttributes']['remap_bool'] = self._data.remap_bool
+        _emd_template['DataAttributes']['important_classes'] = self._data.important_classes
+        _emd_template['DataAttributes']['idx2class'] = self._data.idx2class
+        _emd_template['DataAttributes']['features_to_keep'] = self._data.features_to_keep
+        _emd_template['DataAttributes']['pc_type'] = self._data.pc_type
+        _emd_template['DataAttributes']['background_classcode'] = self._data.background_classcode
+        if self._data.pc_type == 'PointCloud_TF':
+            _emd_template['DataAttributes']['extra_feat_indexes'] = self._data.extra_feat_indexes
+
 
         _emd_template['Classes'] = []
         class_data = {}
-        for i, class_name in enumerate(self._data.classes):  # 0th index is background
+        for i, class_name in enumerate(self._data.classes):
             inverse_class_mapping = {v: k for k, v in self._data.class_mapping.items()}
-            class_data["Value"] = inverse_class_mapping[i]
-            class_data["Name"] = class_name
+            class_data["Value"] = class_name
+            class_data["Name"] = self._data.class_mapping[class_name]
             color = [random.choice(range(256)) for i in range(3)] if is_no_color(self._data.color_mapping) else \
                 self._data.color_mapping[class_name]
             class_data["Color"] = np.array(color).astype(int).tolist()
@@ -429,5 +486,115 @@ class PointCNN(ArcGISModel):
         :returns: Path where files are dumped.
         """
         
-        return predict_h5(self, path, output_path)        
+        return predict_h5(self, path, output_path, **kwargs)
+
+    def _get_name_path(self, name_or_path):
+        ispath_like = '\\' in name_or_path or '/' in name_or_path
+        if ispath_like:
+            path = Path(name_or_path)
+            # to make fastai from both path and with name
+            if path.is_file():
+                name = path.stem
+            else:
+                name = path.parts[-1]
+        else:
+            name = name_or_path
+
+        return name, ispath_like
+
+    def load(self, name_or_path):
+        """
+        Loads a compatible saved model for inferencing or fine tuning from the disk.
+
+        =====================   ===========================================
+        **Argument**            **Description**
+        ---------------------   -------------------------------------------
+        name_or_path            Required string. Name or Path to
+                                Deep Learning Package (DLPK) or
+                                Esri Model Definition(EMD) file.
+        =====================   ===========================================
+        """
+        name, ispath_like = self._get_name_path(name_or_path)
+
+        if not Path(name_or_path).is_absolute() and ispath_like:
+            name_or_path = str(Path(name_or_path).absolute())
+
+        if Path(name_or_path).is_absolute() and name_or_path.endswith('.emd'):
+            emd_path = Path(name_or_path)
+        elif Path(name_or_path).is_absolute() and name_or_path.endswith('.pth'):
+            emd_path = Path(name_or_path).with_suffix('.emd')
+        else:
+            emd_path = self.learn.path / 'models' / name / f'{name}.emd'
+
+        # from_model
+        if not emd_path.exists():
+            super().load(name_or_path)
+            return
+        with open(emd_path) as f:
+            emd = json.load(f)
+
+        try:
+            class_mapping = {i['Value'] : i['Name'] for i in emd['Classes']}
+            color_mapping = {i['Value'] : i['Color'] for i in emd['Classes']}
+        except KeyError:
+            class_mapping = {i['ClassValue'] : i['ClassName'] for i in emd['Classes']} 
+            color_mapping = {i['ClassValue'] : i['Color'] for i in emd['Classes']}   
+
+        dummy_data = _EmptyData(path=emd_path.parent, loss_func=None, c=len(class_mapping), chip_size=emd['ImageHeight'])
+        dummy_data.emd_path = emd_path
+        dummy_data.emd = emd
+        for key, value in emd['DataAttributes'].items():
+            setattr(dummy_data, key, value)
+
+        # old models
+        if not hasattr(dummy_data, 'classes'):
+            dummy_data.classes = [k for k in list(class_mapping.keys())]
+
+        if not hasattr(dummy_data, 'pc_type'):
+            dummy_data.pc_type = 'PointCloud_TF'
+
+        if not hasattr(self._data, 'pc_type'):
+            self._data.pc_type = 'PointCloud_TF'
+
+        string_mapped_features = {
+            'numberOfReturns': 'num_returns',
+            'returnNumber' : 'return_num',
+            'nearInfrared': 'nir'
+        }
+        inverse_string_mapped_features = {v: k for k, v in string_mapped_features.items()} 
+
+        if not hasattr(dummy_data, 'features_to_keep'):
+            dummy_data.features_to_keep = [inverse_string_mapped_features.get(f[0], f[0]) for f in dummy_data.extra_features]
+
+        if not hasattr(self._data, 'features_to_keep'):
+            self._data.features_to_keep = [inverse_string_mapped_features.get(f[0], f[0]) for f in self._data.extra_features]
+
+        # for message
+        api_fn_name = '`export_point_dataset` function'
+        tool_name = '`Prepare Point Cloud Training Data` tool'
+        exported_by = api_fn_name if  dummy_data.pc_type == 'PointCloud_TF' else tool_name
+        training_on = api_fn_name if  self._data.pc_type == 'PointCloud_TF' else tool_name
+
+        if dummy_data.pc_type != self._data.pc_type:
+            raise Exception("Models trained on one type of exported data cannot be trained on other. "
+                            f"Model was trained on exported data from {exported_by}. "
+                            f"Usage with data exported by {training_on} is not supported. "
+                            f"Export the data with {exported_by} for fine-tuning."
+            )
+
+        if dummy_data.max_point != self._data.max_point:
+            raise Exception("Max points do not match. Please export the data again "
+                            f"and set max points to same as loaded model i.e. {dummy_data.max_point}."
+                            )
+
+        if dummy_data.features_to_keep != self._data.features_to_keep:
+            if 'xyz' in dummy_data.features_to_keep: dummy_data.features_to_keep.remove('xyz')
+            raise Exception(f"Extra features of your data and the model to be loaded do not match. "
+                            f"Set `extra_features` attribute in `prepare_data` to be {dummy_data.features_to_keep}"
+                            )
+
+        if dummy_data.classes != self._data.classes:
+            raise Exception(f"Classes in your data and loaded model do not match. Got {self._data.classes}, required {dummy_data.classes}")
+
+        super().load(name_or_path)
         
