@@ -28,6 +28,8 @@ try:
     from fastai.vision import flatten_model
     from ._deeplab_utils import compute_miou
     from ._PointRend import PointRend_target_transform
+    from .._utils.env import _IS_ARCGISPRONOTEBOOK
+    from matplotlib import pyplot as plt
     HAS_FASTAI = True
 except Exception as e:
     HAS_FASTAI = False
@@ -94,7 +96,22 @@ class PSPNetClassifier(ArcGISModel):
                             E.g. 
                             for dice = 0.3, loss = (1-0.3)*default loss + 0.3*dice
                             Default: 0
-    ---------------------   -------------------------------------------    
+    ---------------------   -------------------------------------------
+    dice_loss_average       Optional str. 
+                            micro: Micro dice coefficient will be used for loss 
+                            calculation.
+                            macro: Macro dice coefficient will be used for loss 
+                            calculation.
+                            A macro-average will compute the metric independently 
+                            for each class and then take the average (hence treating 
+                            all classes equally), whereas a micro-average will 
+                            aggregate the contributions of all classes to compute the 
+                            average metric. In a multi-class classification setup, 
+                            micro-average is preferable if you suspect there might be 
+                            class imbalance (i.e you may have many more examples of 
+                            one class than of other classes)
+                            Default: 'micro'
+    ---------------------   -------------------------------------------                        
     ignore_classes          Optional list. It will contain the list of class
                             values on which model will not incur loss.
                             Default: []
@@ -141,6 +158,7 @@ class PSPNetClassifier(ArcGISModel):
         self.focal_loss = kwargs.get('focal_loss', False)
         self.dice_loss_fraction = kwargs.get('dice_loss_fraction', False)
         self.weighted_dice = kwargs.get('weighted_dice', False)
+        self.dice_loss_average = kwargs.get('dice_loss_average', 'micro')
         self.keep_dilation = kwargs.get('keep_dilation', False)
         self._vggv2 = kwargs.get('vggv2', True)
         self._code = image_classifier_prf
@@ -210,7 +228,7 @@ class PSPNetClassifier(ArcGISModel):
         if self.focal_loss:
             self.learn.loss_func = FocalLoss(self.learn.loss_func)            
         if self.dice_loss_fraction:
-            self.learn.loss_func = DiceLoss(self.learn.loss_func, self.dice_loss_fraction,  weighted_dice=self.weighted_dice)
+            self.learn.loss_func = DiceLoss(self.learn.loss_func, self.dice_loss_fraction,  weighted_dice=self.weighted_dice, dice_average = self.dice_loss_average)
         self.learn.model = self.learn.model.to(self._device)
         self.freeze()
         self._arcgis_init_callback() # make first conv weights learnable
@@ -393,6 +411,8 @@ class PSPNetClassifier(ArcGISModel):
         if rows > len(self._data.valid_ds):
             rows = len(self._data.valid_ds)
         self.learn.show_results(rows=rows, ignore_mapped_class=self._ignore_mapped_class, **kwargs)
+        if _IS_ARCGISPRONOTEBOOK:
+            plt.show()
 
     def _show_results_multispectral(self, rows=5, alpha=0.7, **kwargs): # parameters adjusted in kwargs
         return_fig = kwargs.get('return_fig', False)
