@@ -1,4 +1,4 @@
-from ._arcgis_model import ArcGISModel
+from ._arcgis_model import ArcGISModel, _get_device
 from pathlib import Path
 import json
 from ._codetemplate import code
@@ -34,6 +34,8 @@ try:
     from .._video_utils import VideoUtils
     from .._utils.common import get_multispectral_data_params_from_emd, _get_emd_path
     from fastprogress.fastprogress import progress_bar
+    from .._utils.env import _IS_ARCGISPRONOTEBOOK
+    import matplotlib.pyplot as plt
 except Exception as e:
     import_exception = "\n".join(traceback.format_exception(type(e), e, e.__traceback__))
     HAS_FASTAI = False
@@ -117,7 +119,6 @@ class RetinaNet(ArcGISModel):
         if pretrained_path is not None:
             self.load(str(pretrained_path))
         self._arcgis_init_callback() # make first conv weights learnable
-        self.learn.model = self.learn.model.to(self._device)
 
     def __str__(self):
         return self.__repr__()
@@ -222,8 +223,8 @@ class RetinaNet(ArcGISModel):
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", UserWarning)
                 
-                sd = ImageList([], path=emd_path.parent.parent).split_by_idx([])
-                data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch().normalize(imagenet_stats)
+                sd = ImageList([], path=emd_path.parent.parent.parent).split_by_idx([])
+                data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch(device=_get_device()).normalize(imagenet_stats)
 
             data.chip_size = chip_size
             data.class_mapping = class_mapping
@@ -270,6 +271,8 @@ class RetinaNet(ArcGISModel):
             rows = len(self._data.valid_ds)
 
         self.learn.show_results(rows=rows, thresh=thresh, nms_overlap=nms_overlap, model=self)
+        if _IS_ARCGISPRONOTEBOOK:
+            plt.show()
 
     def _show_results_multispectral(self, rows=5, thresh=0.3, nms_overlap=0.1, alpha=1, **kwargs):
         return_fig = kwargs.get('return_fig', False)
