@@ -1,4 +1,4 @@
-from ._arcgis_model import ArcGISModel
+from ._arcgis_model import ArcGISModel, _get_device
 from pathlib import Path
 import json
 from ._codetemplate import code
@@ -40,6 +40,8 @@ try:
     from .._video_utils import VideoUtils
     from .._utils.common import get_multispectral_data_params_from_emd, _get_emd_path, read_image
     from fastprogress.fastprogress import progress_bar
+    from .._utils.env import _IS_ARCGISPRONOTEBOOK
+    import matplotlib.pyplot as plt
 except Exception as e:
     import_exception = "\n".join(traceback.format_exception(type(e), e, e.__traceback__))
     class NnModule():
@@ -215,7 +217,6 @@ class SingleShotDetector(ArcGISModel):
 
             self.learn = cnn_learner(data=data, base_arch=self._backbone, cut=backbone_cut, split_on=backbone_split, custom_head=ssd_head)
             self._arcgis_init_callback() # make first conv weights learnable
-            self.learn.model = self.learn.model.to(self._device)
 
             if focal_loss:
                 self._loss_f = FocalLoss(data.c)
@@ -328,8 +329,8 @@ class SingleShotDetector(ArcGISModel):
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", UserWarning)
                 
-                sd = ImageList([], path=emd_path.parent.parent).split_by_idx([])
-                data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch().normalize(imagenet_stats)
+                sd = ImageList([], path=emd_path.parent.parent.parent).split_by_idx([])
+                data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch(device=_get_device()).normalize(imagenet_stats)
 
             data.chip_size = chip_size
             data.class_mapping = class_mapping
@@ -502,6 +503,8 @@ class SingleShotDetector(ArcGISModel):
         if rows > len(self._data.valid_ds):
             rows = len(self._data.valid_ds)
         self.learn.show_results(rows=rows, thresh=thresh, nms_overlap=nms_overlap, model=self)
+        if _IS_ARCGISPRONOTEBOOK:
+            plt.show()
 
     def _show_results_multispectral(self, rows=5, thresh=0.3, nms_overlap=0.1, alpha=1, **kwargs):
         return_fig = kwargs.get('return_fig', False)
