@@ -116,7 +116,7 @@ class TimeSeriesModel(ArcGISModel):
                 model = model_arch_ob(data_bunch.features, data_bunch.c, **kwargs).to(self._device)
                 if model_arch.lower() in ['resnet', 'fcn']:
                     del kwargs['device']
-            self.learn = Learner(data_bunch, model, model_dir=tempfile.TemporaryDirectory().name)
+            self.learn = Learner(data_bunch, model, path=data.path)
             self.learn.data = data_bunch
 
         self.learn.layer_groups = split_model_idx(self.learn.model, [1])
@@ -233,9 +233,11 @@ class TimeSeriesModel(ArcGISModel):
         """
 
         if '\\' in name_or_path or '/' in name_or_path:
-            path = name_or_path
+            path = os.path.abspath(name_or_path)
         else:
-            path = os.path.join(os.getcwd(), name_or_path)
+            path = os.path.join(self._data.path, 'models', name_or_path)
+            if not os.path.exists(os.path.dirname(path)):
+                os.mkdir(os.path.dirname(path))
 
         if not os.path.exists(path):
             os.mkdir(path)
@@ -554,7 +556,7 @@ class TimeSeriesModel(ArcGISModel):
             for i in range(number_of_predictions):
                 orig_dataframe = orig_dataframe.append(pd.Series(), ignore_index=True)
                 if delta is not None:
-                    orig_dataframe[index_field_name].iloc[len(orig_dataframe)-1] = end_value + delta.to_timedelta64()
+                    orig_dataframe.loc[len(orig_dataframe)-1, index_field_name] = end_value + delta.to_timedelta64()
                     end_value = end_value + delta.to_timedelta64()
 
         if match_field_names and match_field_names.get(self._data._dependent_variable):
@@ -680,7 +682,19 @@ class TimeSeriesModel(ArcGISModel):
         if prediction_type == "dataframe":
             return orig_dataframe
 
-        return orig_dataframe.spatial.to_featurelayer(output_layer_name, gis)
+        if 'SHAPE' in list(orig_dataframe.columns):
+            orig_dataframe.spatial.to_featurelayer(output_layer_name, gis)
+        else:
+            import tempfile
+            try:
+                import openpyxl
+            except:
+                warnings.warn("This environment does not have openpyxl installed. Please install this dependency using the command: pip install openpyxl")
+            with tempfile.TemporaryDirectory() as tmpdir:
+                table_file = os.path.join(tmpdir, output_layer_name + '.xlsx')
+                orig_dataframe.to_excel(table_file, index=False, header=True)
+                online_table = gis.content.add({'type': 'Microsoft Excel', 'overwrite': True}, table_file)
+                return online_table.publish(overwrite=True)
 
     def score(self):
         """
