@@ -26,6 +26,8 @@ try:
     from ._psp_utils import accuracy
     from ._deeplab_utils import compute_miou
     import os as arcgis_os
+    from matplotlib import pyplot as plt
+    from .._utils.env import _IS_ARCGISPRONOTEBOOK
     HAS_FASTAI = True
 except Exception as e:
     import_exception = "\n".join(traceback.format_exception(type(e), e, e.__traceback__))
@@ -81,6 +83,21 @@ class UnetClassifier(ArcGISModel):
                             for dice = 0.3, loss = (1-0.3)*default loss + 0.3*dice
                             Default: 0
     ---------------------   -------------------------------------------
+    dice_loss_average       Optional str. 
+                            micro: Micro dice coefficient will be used for loss 
+                            calculation.
+                            macro: Macro dice coefficient will be used for loss 
+                            calculation.
+                            A macro-average will compute the metric independently 
+                            for each class and then take the average (hence treating 
+                            all classes equally), whereas a micro-average will 
+                            aggregate the contributions of all classes to compute the 
+                            average metric. In a multi-class classification setup, 
+                            micro-average is preferable if you suspect there might be 
+                            class imbalance (i.e you may have many more examples of 
+                            one class than of other classes)
+                            Default: 'micro'
+    ---------------------   -------------------------------------------                        
     ignore_classes          Optional list. It will contain the list of class
                             values on which model will not incur loss.
                             Default: []
@@ -122,6 +139,7 @@ class UnetClassifier(ArcGISModel):
             self.focal_loss = kwargs.get('focal_loss', False)
             self.dice_loss_fraction = kwargs.get('dice_loss_fraction', False)
             self.weighted_dice = kwargs.get('weighted_dice', False)
+            self.dice_loss_average = kwargs.get('dice_loss_average', 'micro')
 
             self._code = image_classifier_prf
 
@@ -172,7 +190,7 @@ class UnetClassifier(ArcGISModel):
                 self.learn.callbacks.append(MixUpCallback(self.learn))
 
             if self.dice_loss_fraction:
-                self.learn.loss_func = DiceLoss(self.learn.loss_func, self.dice_loss_fraction,  weighted_dice=self.weighted_dice)
+                self.learn.loss_func = DiceLoss(self.learn.loss_func, self.dice_loss_fraction,  weighted_dice=self.weighted_dice, dice_average = self.dice_loss_average)
 
             self._arcgis_init_callback() # make first conv weights learnable
             self.learn.callbacks.append(LabelCallback(self.learn))  #appending label callback
@@ -337,6 +355,8 @@ class UnetClassifier(ArcGISModel):
         if rows > len(self._data.valid_ds):
             rows = len(self._data.valid_ds)
         self.learn.show_results(rows=rows, ignore_mapped_class=self._ignore_mapped_class, **kwargs)
+        if _IS_ARCGISPRONOTEBOOK:
+            plt.show()
 
     def accuracy(self):
         try:
