@@ -8,7 +8,7 @@ import statistics
 from pathlib import Path
 from ._codetemplate import code
 from .._data import _raise_fastai_import_error
-from ._arcgis_model import ArcGISModel
+from ._arcgis_model import ArcGISModel, _get_device
 
 HAS_OPENCV = True
 HAS_FASTAI = True
@@ -29,6 +29,7 @@ try:
     from ._yolov3_utils import download_yolo_weights, parse_yolo_weights, postprocess, coco_config, coco_class_mapping
     from .._image_utils import _get_image_chips, _get_transformed_predictions, _draw_predictions, _exclude_detection
     from .._video_utils import VideoUtils
+    from .._utils.env import _IS_ARCGISPRONOTEBOOK
 except Exception as e:
     import_exception = "\n".join(traceback.format_exception(type(e), e, e.__traceback__))
     HAS_FASTAI = False
@@ -193,6 +194,9 @@ class YOLOv3(ArcGISModel):
 
         self.learn.predicting = True
         self.learn.show_results(rows=rows, thresh=thresh, nms_overlap=nms_overlap, model=self)
+        if _IS_ARCGISPRONOTEBOOK:
+            import matplotlib.pyplot as plt
+            plt.show()
     
     def _show_results_multispectral(self, rows=5, thresh=0.1, nms_overlap=0.1, alpha=1, **kwargs):
         return_fig = kwargs.get('return_fig', False)
@@ -590,8 +594,8 @@ class YOLOv3(ArcGISModel):
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", UserWarning)
                 
-                sd = ImageList([], path=emd_path.parent.parent).split_by_idx([])
-                data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch().normalize(imagenet_stats)
+                sd = ImageList([], path=emd_path.parent.parent.parent).split_by_idx([])
+                data = sd.label_const(0, label_cls=ObjectDetectionCategoryList, classes=list(class_mapping.values())).transform(ds_tfms).databunch(device=_get_device()).normalize(imagenet_stats)
 
             data.chip_size = chip_size
             data.class_mapping = class_mapping
